@@ -22,6 +22,7 @@ from openmdao.core.constants import _UNDEFINED
 import openmdao.core.total_jac as tot_jac_mod
 
 from openmdao.utils.mpi import MPI
+from openmdao.utils.rich_utils import strip_formatting
 
 try:
     from openmdao.vectors.petsc_vector import PETScVector
@@ -412,29 +413,30 @@ class TestProblemCheckTotals(unittest.TestCase):
         # check derivatives with complex step and a larger step size.
         stream = StringIO()
         totals = prob.check_totals(method='cs', out_stream=stream)
+        assert_check_totals(totals)
 
         lines = stream.getvalue().splitlines()
 
         # Make sure auto-ivc sources are translated to promoted input names.
         self.assertTrue('x' in lines[4])
 
-        self.assertTrue('9.80614' in lines[5], "'9.80614' not found in '%s'" % lines[5])
-        self.assertTrue('9.80614' in lines[6], "'9.80614' not found in '%s'" % lines[6])
-        self.assertTrue('cs:None' in lines[6], "'cs:None not found in '%s'" % lines[6])
+        self.assertTrue('-9.80614' in lines[9], "'-9.80614' not found in '%s'" % lines[9])
+        self.assertTrue('-9.80614' in lines[10], "'-9.80614' not found in '%s'" % lines[10])
+        self.assertTrue('cs:None' in lines[10], "'cs:None not found in '%s'" % lines[10])
 
         assert_near_equal(totals['con2', 'x']['J_fwd'], [[0.09692762]], 1e-5)
         assert_near_equal(totals['con2', 'x']['J_fd'], [[0.09692762]], 1e-5)
 
         # Test compact_print output
         compact_stream = StringIO()
-        prob.check_totals(method='fd', out_stream=compact_stream, compact_print=True)
+        assert_check_totals(prob.check_totals(method='cs', out_stream=compact_stream, compact_print=True))
 
         compact_lines = compact_stream.getvalue().splitlines()
 
-        self.assertTrue("of '<variable>'" in compact_lines[5],
-            "of '<variable>' not found in '%s'" % compact_lines[5])
-        self.assertTrue('9.7743e+00' in compact_lines[-2],
-            "'9.7743e+00' not found in '%s'" % compact_lines[-2])
+        self.assertTrue("'of' variable" in compact_lines[5],
+            "'of' variable not found in '%s'" % compact_lines[5])
+        self.assertTrue('-9.8061' in compact_lines[7],
+            "'-9.8061' not found in '%s'" % compact_lines[7])
 
     def test_check_totals_show_progress(self):
         prob = om.Problem()
@@ -500,10 +502,9 @@ class TestProblemCheckTotals(unittest.TestCase):
 
         lines = stream.getvalue().splitlines()
 
-        self.assertTrue('1.000' in lines[5])
-        self.assertTrue('1.000' in lines[6])
-        self.assertTrue('0.000' in lines[8])
-        self.assertTrue('0.000' in lines[10])
+        self.assertTrue('1.000' in lines[9])
+        self.assertTrue('1.000' in lines[10])
+        self.assertFalse(lines[6].strip().endswith('*'))
 
         assert_near_equal(totals['x', 'x']['J_fwd'], [[1.0]], 1e-5)
         assert_near_equal(totals['x', 'x']['J_fd'], [[1.0]], 1e-5)
@@ -567,6 +568,8 @@ class TestProblemCheckTotals(unittest.TestCase):
         assert_near_equal(J['y1', 'x1'][1][1], Jbase[2, 3], 1e-8)
 
         totals = prob.check_totals()
+        assert_check_totals(totals)
+
         jac = totals[('y1', 'x1')]['J_fd']
         assert_near_equal(jac[0][0], Jbase[0, 1], 1e-8)
         assert_near_equal(jac[0][1], Jbase[0, 3], 1e-8)
@@ -602,6 +605,7 @@ class TestProblemCheckTotals(unittest.TestCase):
         assert_near_equal(J['y1', 'x1'][0][1], Jbase[1, 3], 1e-8)
 
         totals = prob.check_totals()
+        assert_check_totals(totals)
         jac = totals[('y1', 'x1')]['J_fd']
         assert_near_equal(jac[0][0], Jbase[1, 1], 1e-8)
         assert_near_equal(jac[0][1], Jbase[1, 3], 1e-8)
@@ -633,8 +637,7 @@ class TestProblemCheckTotals(unittest.TestCase):
 
         data = totals['con2', 'x']
         self.assertTrue('J_fwd' in data)
-        self.assertTrue('rel error' in data)
-        self.assertTrue('abs error' in data)
+        self.assertTrue('tol violation' in data)
         self.assertTrue('magnitude' in data)
 
     def test_two_desvar_as_con(self):
@@ -656,15 +659,7 @@ class TestProblemCheckTotals(unittest.TestCase):
         prob.run_model()
 
         totals = prob.check_totals(method='fd', step=1.0e-1, out_stream=None)
-
-        assert_near_equal(totals['x', 'x']['J_fwd'], [[1.0]], 1e-5)
-        assert_near_equal(totals['x', 'x']['J_fd'], [[1.0]], 1e-5)
-        assert_near_equal(totals['z', 'z']['J_fwd'], np.eye(2), 1e-5)
-        assert_near_equal(totals['z', 'z']['J_fd'], np.eye(2), 1e-5)
-        assert_near_equal(totals['x', 'z']['J_fwd'], [[0.0, 0.0]], 1e-5)
-        assert_near_equal(totals['x', 'z']['J_fd'], [[0.0, 0.0]], 1e-5)
-        assert_near_equal(totals['z', 'x']['J_fwd'], [[0.0], [0.0]], 1e-5)
-        assert_near_equal(totals['z', 'x']['J_fd'], [[0.0], [0.0]], 1e-5)
+        assert_check_totals(totals)
 
     def test_full_con_with_index_desvar(self):
         prob = om.Problem()
@@ -683,9 +678,7 @@ class TestProblemCheckTotals(unittest.TestCase):
         prob.run_model()
 
         totals = prob.check_totals(method='fd', step=1.0e-1, out_stream=None)
-
-        assert_near_equal(totals['z', 'z']['J_fwd'], [[0.0], [1.0]], 1e-5)
-        assert_near_equal(totals['z', 'z']['J_fd'], [[0.0], [1.0]], 1e-5)
+        assert_check_totals(totals)
 
     def test_full_desvar_with_index_con(self):
         prob = om.Problem()
@@ -704,9 +697,7 @@ class TestProblemCheckTotals(unittest.TestCase):
         prob.run_model()
 
         totals = prob.check_totals(method='fd', step=1.0e-1, out_stream=None)
-
-        assert_near_equal(totals['z', 'z']['J_rev'], [[0.0, 1.0]], 1e-5)
-        assert_near_equal(totals['z', 'z']['J_fd'], [[0.0, 1.0]], 1e-5)
+        assert_check_totals(totals)
 
     def test_full_desvar_with_index_obj(self):
         prob = om.Problem()
@@ -725,9 +716,7 @@ class TestProblemCheckTotals(unittest.TestCase):
         prob.run_model()
 
         totals = prob.check_totals(method='fd', step=1.0e-1, out_stream=None)
-
-        assert_near_equal(totals['z', 'z']['J_rev'], [[0.0, 1.0]], 1e-5)
-        assert_near_equal(totals['z', 'z']['J_fd'], [[0.0, 1.0]], 1e-5)
+        assert_check_totals(totals)
 
     def test_bug_fd_with_sparse(self):
         # This bug was found via the x57 model in pointer.
@@ -813,10 +802,8 @@ class TestProblemCheckTotals(unittest.TestCase):
         p.run_model()
 
         # Make sure we don't bomb out with an error.
-        J = p.check_totals(out_stream=None)
-
-        assert_near_equal(J[('time', 't_duration')]['J_fwd'][0], 17.0, 1e-5)
-        assert_near_equal(J[('time', 't_duration')]['J_fd'][0], 17.0, 1e-5)
+        totals = p.check_totals(out_stream=None)
+        assert_check_totals(totals)
 
         # Try again with a direct solver and sparse assembled hierarchy.
 
@@ -832,16 +819,12 @@ class TestProblemCheckTotals(unittest.TestCase):
         p.run_model()
 
         # Make sure we don't bomb out with an error.
-        J = p.check_totals(out_stream=None)
-
-        assert_near_equal(J[('sub.time', 'sub.t_duration')]['J_fwd'][0], 17.0, 1e-5)
-        assert_near_equal(J[('sub.time', 'sub.t_duration')]['J_fd'][0], 17.0, 1e-5)
+        totals = p.check_totals(out_stream=None)
+        assert_check_totals(totals)
 
         # Make sure check_totals cleans up after itself by running it a second time
-        J = p.check_totals(out_stream=None)
-
-        assert_near_equal(J[('sub.time', 'sub.t_duration')]['J_fwd'][0], 17.0, 1e-5)
-        assert_near_equal(J[('sub.time', 'sub.t_duration')]['J_fd'][0], 17.0, 1e-5)
+        totals = p.check_totals(out_stream=None)
+        assert_check_totals(totals)
 
     def test_vector_scaled_derivs(self):
 
@@ -879,7 +862,7 @@ class TestProblemCheckTotals(unittest.TestCase):
         assert_near_equal(J, derivs['comp.y1']['px.x'], 1.0e-3)
 
         cderiv = prob.check_totals(driver_scaling=True, out_stream=None)
-        assert_near_equal(cderiv['comp.y1', 'px.x']['J_fwd'], J, 1.0e-3)
+        assert_check_totals(cderiv)
 
         # cleanup after FD
         prob.run_model()
@@ -892,7 +875,7 @@ class TestProblemCheckTotals(unittest.TestCase):
         assert_near_equal(J, derivs['comp.y1']['px.x'], 1.0e-3)
 
         cderiv = prob.check_totals(out_stream=None)
-        assert_near_equal(cderiv['comp.y1', 'px.x']['J_fwd'], J, 1.0e-3)
+        assert_check_totals(cderiv)
 
     def test_cs_around_newton(self):
         # Basic sellar test.
@@ -932,9 +915,7 @@ class TestProblemCheckTotals(unittest.TestCase):
         prob.run_model()
 
         totals = prob.check_totals(method='cs', out_stream=None)
-
-        for key, val in totals.items():
-            assert_near_equal(val['rel error'][0], 0.0, 1e-10)
+        assert_check_totals(totals)
 
     def test_cs_around_newton_new_method(self):
         # The old method of nudging the Newton and forcing it to reconverge could not achieve the
@@ -1066,9 +1047,7 @@ class TestProblemCheckTotals(unittest.TestCase):
 
         prob.run_model()
         totals = prob.check_totals(method='cs', out_stream=None)
-
-        for key, val in totals.items():
-            assert_near_equal(val['rel error'][0], 0.0, 1e-12)
+        assert_check_totals(totals)
 
     def test_cs_around_broyden(self):
         # Basic sellar test.
@@ -1108,9 +1087,7 @@ class TestProblemCheckTotals(unittest.TestCase):
         prob.run_model()
 
         totals = prob.check_totals(method='cs', out_stream=None)
-
-        for key, val in totals.items():
-            assert_near_equal(val['rel error'][0], 0.0, 1e-6)
+        assert_check_totals(totals)
 
     def test_cs_around_newton_top_sparse(self):
         prob = om.Problem()
@@ -1163,8 +1140,8 @@ class TestProblemCheckTotals(unittest.TestCase):
         prob.setup()
         prob.run_model()
 
-        msg = "\nProblem .*: To enable complex step, specify 'force_alloc_complex=True' when calling " + \
-                "setup on the problem, e\.g\. 'problem\.setup\(force_alloc_complex=True\)'"
+        msg = r"\nProblem .*: To enable complex step, specify 'force_alloc_complex=True' when calling " + \
+                r"setup on the problem, e\.g\. 'problem\.setup\(force_alloc_complex=True\)'"
         with self.assertRaisesRegex(RuntimeError, msg):
             prob.check_totals(method='cs')
 
@@ -1198,9 +1175,7 @@ class TestProblemCheckTotals(unittest.TestCase):
         prob.run_model()
 
         # This test verifies fix of a TypeError (division by None)
-        J = prob.check_totals(out_stream=None)
-        assert_near_equal(J['comp.y', 'p.x']['J_fwd'], [[14.0]], 1e-6)
-        assert_near_equal(J['comp.y', 'p.x']['J_fd'], [[0.0]], 1e-6)
+        prob.check_totals(out_stream=None)
 
     def test_response_index(self):
         prob = om.Problem()
@@ -1217,7 +1192,7 @@ class TestProblemCheckTotals(unittest.TestCase):
         prob.run_model()
 
         stream = StringIO()
-        prob.check_totals(out_stream=stream)
+        assert_check_totals(prob.check_totals(out_stream=stream))
         lines = stream.getvalue().splitlines()
         self.assertTrue('index size: 1' in lines[4])
 
@@ -1239,15 +1214,11 @@ class TestProblemCheckTotals(unittest.TestCase):
         p.run_model()
 
         stream = StringIO()
-        J_driver = p.check_totals(out_stream=stream)
-        lines = stream.getvalue().splitlines()
+        assert_check_totals(p.check_totals(out_stream=stream))
+        lines = [strip_formatting(s) for s in stream.getvalue().splitlines()]
 
         self.assertTrue("Full Model: 'lcy' wrt 'x' (Linear constraint)" in lines[4])
-        self.assertTrue("Absolute Error (Jfor - Jfd)" in lines[8])
-        self.assertTrue("Relative Error (Jfor - Jfd) / Jfd" in lines[10])
-
-        assert_near_equal(J_driver['y', 'x']['J_fwd'][0, 0], 1.0)
-        assert_near_equal(J_driver['lcy', 'x']['J_fwd'][0, 0], 3.0)
+        self.assertTrue("Max Tolerance Violation (Jfwd - Jfd) - (atol + rtol * Jfd)" in lines[6])
 
     def test_alias_constraints(self):
         prob = om.Problem()
@@ -1273,11 +1244,7 @@ class TestProblemCheckTotals(unittest.TestCase):
         prob.run_driver()
 
         totals = prob.check_totals(out_stream=None)
-
-        assert_near_equal(totals['areas', 'widths']['abs error'][0], 0.0, 1e-6)
-        assert_near_equal(totals['a2', 'widths']['abs error'][0], 0.0, 1e-6)
-        assert_near_equal(totals['a3', 'widths']['abs error'][0], 0.0, 1e-6)
-        assert_near_equal(totals['a4', 'widths']['abs error'][0], 0.0, 1e-6)
+        assert_check_totals(totals)
 
         prob.list_driver_vars(show_promoted_name=True, print_arrays=False,
                               cons_opts=['indices', 'alias'])
@@ -1307,11 +1274,7 @@ class TestProblemCheckTotals(unittest.TestCase):
         prob.run_driver()
 
         totals = prob.check_totals(out_stream=None)
-
-        assert_near_equal(totals['areas', 'widths']['abs error'][1], 0.0, 1e-6)
-        assert_near_equal(totals['a2', 'widths']['abs error'][1], 0.0, 1e-6)
-        assert_near_equal(totals['a3', 'widths']['abs error'][1], 0.0, 1e-6)
-        assert_near_equal(totals['a4', 'widths']['abs error'][1], 0.0, 1e-6)
+        assert_check_totals(totals)
 
     def test_alias_constraints_nested(self):
         # Tests a bug where we need to lookup the constraint alias on a response that is from
@@ -1365,14 +1328,8 @@ class TestProblemCheckTotals(unittest.TestCase):
         prob.compute_totals()
 
         stream = StringIO()
-        prob.check_totals(out_stream=stream, show_only_incorrect=True)
-
-        self.assertEqual(stream.getvalue().count("'C2.y' wrt 'badcomp.y1'"), 1)
-        self.assertEqual(stream.getvalue().count("'C2.y' wrt 'badcomp.y2'"), 1)
-        self.assertEqual(stream.getvalue().count("'C1.y' wrt 'goodcomp.y1'"), 0)
-        self.assertEqual(stream.getvalue().count("'C1.y' wrt 'goodcomp.y2'"), 0)
-        self.assertEqual(stream.getvalue().count("'C2.y' wrt 'goodcomp.y1'"), 0)
-        self.assertEqual(stream.getvalue().count("'C2.y' wrt 'goodcomp.y2'"), 0)
+        totals = prob.check_totals(out_stream=stream, show_only_incorrect=True)
+        assert_check_totals(totals)
 
     def test_compact_print_exceed_tol_show_only_incorrect(self):
 
@@ -1401,8 +1358,7 @@ class TestProblemCheckTotals(unittest.TestCase):
         stream = StringIO()
         prob.check_totals(out_stream=stream, show_only_incorrect=True, compact_print=True)
 
-        self.assertEqual(stream.getvalue().count('>ABS_TOL'), 2)
-        self.assertEqual(stream.getvalue().count('>REL_TOL'), 2)
+        self.assertEqual(stream.getvalue().count('>TOL'), 2)
 
     def test_directional_vectorized_matrix_free_fwd(self):
 
@@ -1416,16 +1372,16 @@ class TestProblemCheckTotals(unittest.TestCase):
 
         stream = StringIO()
         data = prob.check_totals(method='cs', out_stream=stream, directional=True)
-        content = stream.getvalue()
+        content = strip_formatting(stream.getvalue())
 
-        self.assertEqual(content.count('Reverse Magnitude:'), 0)
-        self.assertEqual(content.count('Forward Magnitude:'), 1)
-        self.assertEqual(content.count('Fd Magnitude:'), 1)
-        self.assertEqual(content.count('Directional Derivative (Jfor)'), 1)
+        self.assertEqual(content.count('rev value:'), 0)
+        self.assertEqual(content.count('fwd value:'), 1)
+        self.assertEqual(content.count('fd value:'), 1)
+        self.assertEqual(content.count('Directional Derivative (Jfwd)'), 1)
         self.assertEqual(content.count('Directional CS Derivative (Jfd)'), 1)
-        self.assertTrue('Relative Error (Jfor - Jfd) / Jfd : ' in content)
-        self.assertTrue('Absolute Error (Jfor - Jfd) : ' in content)
-        assert_near_equal(data[(('comp.out',), 'comp.in')]['directional_fd_fwd'], 0., tolerance=2e-15)
+        self.assertTrue('Max Tolerance Violation ([fwd, fd] Dot Product Test) : ' in content)
+        mhatdotm, dhatdotd =  data[(('comp.out',), 'comp.in')]['directional_fd_fwd']
+        assert_near_equal(mhatdotm, dhatdotd, tolerance=2e-15)
 
     def test_directional_vectorized_matrix_free_rev_index_0(self):
 
@@ -1442,14 +1398,14 @@ class TestProblemCheckTotals(unittest.TestCase):
         content = stream.getvalue()
 
         self.assertEqual(content.count('comp.out (index size: 1)'), 1)
-        self.assertEqual(content.count('Reverse Magnitude:'), 1)
-        self.assertEqual(content.count('Forward Magnitude:'), 0)
-        self.assertEqual(content.count('Fd Magnitude:'), 1)
+        self.assertEqual(content.count('rev value:'), 1)
+        self.assertEqual(content.count('fwd value:'), 0)
+        self.assertEqual(content.count('fd value:'), 1)
         self.assertEqual(content.count('Directional Derivative (Jrev)'), 1)
         self.assertEqual(content.count('Directional CS Derivative (Jfd)'), 1)
-        self.assertTrue('Relative Error ([rev, fd] Dot Product Test) / Jfd : ' in content)
-        self.assertTrue('Absolute Error ([rev, fd] Dot Product Test) : ' in content)
-        assert_near_equal(data[('comp.out', ('comp.in',))]['directional_fd_rev'], 0., tolerance=2e-15)
+        self.assertTrue('Max Tolerance Violation ([rev, fd] Dot Product Test) : ' in content)
+        dJrev, dJfd = data[('comp.out', ('comp.in',))]['directional_fd_rev']
+        assert_near_equal(dJrev - dJfd, 0., tolerance=2e-15)
 
     def test_directional_vectorized_matrix_free_rev(self):
 
@@ -1463,17 +1419,17 @@ class TestProblemCheckTotals(unittest.TestCase):
 
         stream = StringIO()
         data = prob.check_totals(method='cs', out_stream=stream, directional=True)
-        content = stream.getvalue()
+        content = strip_formatting(stream.getvalue())
 
         self.assertEqual(content.count("'comp.out' wrt (d)('comp.in',)"), 1)
-        self.assertEqual(content.count('Reverse Magnitude:'), 1)
-        self.assertEqual(content.count('Forward Magnitude:'), 0)
-        self.assertEqual(content.count('Fd Magnitude:'), 1)
+        self.assertEqual(content.count('rev value:'), 1)
+        self.assertEqual(content.count('fwd value:'), 0)
+        self.assertEqual(content.count('fd value:'), 1)
         self.assertEqual(content.count('Directional Derivative (Jrev)'), 1)
         self.assertEqual(content.count('Directional CS Derivative (Jfd)'), 1)
-        self.assertTrue('Relative Error ([rev, fd] Dot Product Test) / Jfd : ' in content)
-        self.assertTrue('Absolute Error ([rev, fd] Dot Product Test) : ' in content)
-        assert_near_equal(data[('comp.out', ('comp.in',))]['directional_fd_rev'], 0., tolerance=2e-15)
+        self.assertTrue('Max Tolerance Violation ([rev, fd] Dot Product Test) : ' in content)
+        dJrev, dJfd =  data[('comp.out', ('comp.in',))]['directional_fd_rev']
+        assert_near_equal(dJrev - dJfd, 0., tolerance=2e-15)
 
     def test_directional_vectorized_matrix_free_rev_2in2out(self):
 
@@ -1489,19 +1445,20 @@ class TestProblemCheckTotals(unittest.TestCase):
 
         stream = StringIO()
         data = prob.check_totals(method='cs', out_stream=stream, directional=True)
-        content = stream.getvalue()
+        content = strip_formatting(stream.getvalue())
 
         self.assertEqual(content.count("'comp.out1' wrt (d)('comp.in1', 'comp.in2')"), 1)
         self.assertEqual(content.count("'comp.out2' wrt (d)('comp.in1', 'comp.in2')"), 1)
-        self.assertEqual(content.count('Reverse Magnitude:'), 2)
-        self.assertEqual(content.count('Forward Magnitude:'), 0)
-        self.assertEqual(content.count('Fd Magnitude:'), 2)
+        self.assertEqual(content.count('rev value:'), 2)
+        self.assertEqual(content.count('fwd value:'), 0)
+        self.assertEqual(content.count('fd value:'), 2)
         self.assertEqual(content.count('Directional Derivative (Jrev)'), 2)
         self.assertEqual(content.count('Directional CS Derivative (Jfd)'), 2)
-        self.assertTrue(content.count('Relative Error ([rev, fd] Dot Product Test) / Jfd :'), 2)
-        self.assertTrue(content.count('Absolute Error ([rev, fd] Dot Product Test) :'), 2)
-        assert_near_equal(data[('comp.out1', ('comp.in1', 'comp.in2'))]['directional_fd_rev'], 0., tolerance=2e-15)
-        assert_near_equal(data[('comp.out2', ('comp.in1', 'comp.in2'))]['directional_fd_rev'], 0., tolerance=2e-15)
+        self.assertTrue(content.count('Max Tolerance Violation ([rev, fd] Dot Product Test) :'), 2)
+        dJrev, dJfd = data[('comp.out1', ('comp.in1', 'comp.in2'))]['directional_fd_rev']
+        assert_near_equal(dJrev - dJfd, 0., tolerance=2e-15)
+        dJrev, dJfd = data[('comp.out2', ('comp.in1', 'comp.in2'))]['directional_fd_rev']
+        assert_near_equal(dJrev - dJfd, 0., tolerance=2e-15)
 
     def test_directional_vectorized_matrix_free_rev_2in2out_compact(self):
 
@@ -1519,8 +1476,10 @@ class TestProblemCheckTotals(unittest.TestCase):
         data = prob.check_totals(method='cs', out_stream=stream, compact_print=True, directional=True)
         content = stream.getvalue().strip()
         self.assertEqual(content.count("('comp.in1', 'comp.in2')"), 2)
-        assert_near_equal(data[('comp.out1', ('comp.in1', 'comp.in2'))]['directional_fd_rev'], 0., tolerance=2e-15)
-        assert_near_equal(data[('comp.out2', ('comp.in1', 'comp.in2'))]['directional_fd_rev'], 0., tolerance=2e-15)
+        dJrev, dJfd =  data[('comp.out1', ('comp.in1', 'comp.in2'))]['directional_fd_rev']
+        assert_near_equal(dJrev - dJfd, 0., tolerance=2e-15)
+        dJrev, dJfd =  data[('comp.out2', ('comp.in1', 'comp.in2'))]['directional_fd_rev']
+        assert_near_equal(dJrev - dJfd, 0., tolerance=2e-15)
 
     def test_directional_vectorized_matrix_free_fwd_2in2out(self):
 
@@ -1537,19 +1496,21 @@ class TestProblemCheckTotals(unittest.TestCase):
         stream = StringIO()
         data = prob.check_totals(method='cs', out_stream=stream, directional=True)
         content = stream.getvalue()
+        content = strip_formatting(content)
 
         self.assertEqual(content.count("('comp.out1', 'comp.out2') wrt (d)'comp.in1'"), 1)
         self.assertEqual(content.count("('comp.out1', 'comp.out2') wrt (d)'comp.in2'"), 1)
-        self.assertEqual(content.count('Reverse Magnitude:'), 0)
-        self.assertEqual(content.count('Forward Magnitude:'), 2)
-        self.assertEqual(content.count('Fd Magnitude:'), 2)
+        self.assertEqual(content.count('rev value:'), 0)
+        self.assertEqual(content.count('fwd value:'), 2)
+        self.assertEqual(content.count('fd value:'), 2)
         self.assertEqual(content.count('Directional Derivative (Jrev)'), 0)
-        self.assertEqual(content.count('Directional Derivative (Jfor)'), 2)
+        self.assertEqual(content.count('Directional Derivative (Jfwd)'), 2)
         self.assertEqual(content.count('Directional CS Derivative (Jfd)'), 2)
-        self.assertEqual(content.count('Relative Error ([rev, fd] Dot Product Test) / Jfd :'), 0)
-        self.assertEqual(content.count('Absolute Error ([rev, fd] Dot Product Test) :'), 0)
-        assert_near_equal(np.linalg.norm(data[(('comp.out1', 'comp.out2'), 'comp.in1')]['directional_fd_fwd']), 0., tolerance=2e-15)
-        assert_near_equal(np.linalg.norm(data[(('comp.out1', 'comp.out2'), 'comp.in2')]['directional_fd_fwd']), 0., tolerance=2e-15)
+        self.assertEqual(content.count('Max Tolerance Violation ([rev, fd] Dot Product Test) :'), 0)
+        dJfwd, dJfd =  data[(('comp.out1', 'comp.out2'), 'comp.in1')]['directional_fd_fwd']
+        assert_near_equal(np.linalg.norm(dJfwd - dJfd), 0., tolerance=2e-15)
+        dJfwd, dJfd =  data[(('comp.out1', 'comp.out2'), 'comp.in2')]['directional_fd_fwd']
+        assert_near_equal(np.linalg.norm(dJfwd - dJfd), 0., tolerance=2e-15)
 
     def test_directional_vectorized_matrix_free_fwd_2in2out_compact(self):
 
@@ -1567,8 +1528,10 @@ class TestProblemCheckTotals(unittest.TestCase):
         data = prob.check_totals(method='cs', out_stream=stream, compact_print=True, directional=True)
         content = stream.getvalue().strip()
         self.assertEqual(content.count("('comp.out1', 'comp.out2')"), 2)
-        assert_near_equal(np.linalg.norm(data[(('comp.out1', 'comp.out2'), 'comp.in1')]['directional_fd_fwd']), 0., tolerance=2e-15)
-        assert_near_equal(np.linalg.norm(data[(('comp.out1', 'comp.out2'), 'comp.in2')]['directional_fd_fwd']), 0., tolerance=2e-15)
+        dJfwd, dJfd =  data[(('comp.out1', 'comp.out2'), 'comp.in1')]['directional_fd_fwd']
+        assert_near_equal(np.linalg.norm(dJfwd - dJfd), 0., tolerance=2e-15)
+        dJfwd, dJfd =  data[(('comp.out1', 'comp.out2'), 'comp.in2')]['directional_fd_fwd']
+        assert_near_equal(np.linalg.norm(dJfwd - dJfd), 0., tolerance=2e-15)
 
     def test_directional_dymosish(self):
         class CollocationComp(om.ExplicitComponent):
@@ -2036,11 +1999,7 @@ class TestProblemCheckTotalsMPI(unittest.TestCase):
         prob.set_solver_print(level=0)
         prob.run_model()
 
-        J = prob.check_totals(out_stream=None)
-        assert_near_equal(J['sum.y', 'sub.sub1.p1.x']['J_rev'], [[2.0]], 1.0e-6)
-        assert_near_equal(J['sum.y', 'sub.sub2.p2.x']['J_rev'], [[4.0]], 1.0e-6)
-        assert_near_equal(J['sum.y', 'sub.sub1.p1.x']['J_fd'], [[2.0]], 1.0e-6)
-        assert_near_equal(J['sum.y', 'sub.sub2.p2.x']['J_fd'], [[4.0]], 1.0e-6)
+        assert_check_totals(prob.check_totals(out_stream=None))
 
 
 class TestCheckTotalsMultipleSteps(unittest.TestCase):
@@ -2050,13 +2009,12 @@ class TestCheckTotalsMultipleSteps(unittest.TestCase):
         p.run_model()
         stream = StringIO()
         p.check_totals(step=[1e-6], out_stream=stream)
-        contents = stream.getvalue()
+        contents = strip_formatting(stream.getvalue())
         nsubjacs = 18
         self.assertEqual(contents.count("Full Model:"), nsubjacs)
-        self.assertEqual(contents.count("Fd Magnitude:"), nsubjacs)
-        self.assertEqual(contents.count("Absolute Error (Jfor - Jfd), step="), 0)
-        self.assertEqual(contents.count("Absolute Error (Jfor - Jfd)"), nsubjacs)
-        self.assertEqual(contents.count("Relative Error (Jfor - Jfd) / Jf"), nsubjacs)
+        self.assertEqual(contents.count("fd value:"), nsubjacs)
+        self.assertEqual(contents.count("Max Tolerance Violation (Jfwd - Jfd) - (atol + rtol * Jfd), step="), 0)
+        self.assertEqual(contents.count("Max Tolerance Violation (Jfwd - Jfd) - (atol + rtol * Jfd)"), nsubjacs)
         self.assertEqual(contents.count("Raw FD Derivative (Jfd), step="), 0)
         self.assertEqual(contents.count("Raw FD Derivative (Jfd)"), nsubjacs)
 
@@ -2066,13 +2024,12 @@ class TestCheckTotalsMultipleSteps(unittest.TestCase):
         p.run_model()
         stream = StringIO()
         p.check_totals(step=[1e-6], out_stream=stream)
-        contents = stream.getvalue()
+        contents = strip_formatting(stream.getvalue())
         nsubjacs = 18
         self.assertEqual(contents.count("Full Model:"), nsubjacs)
-        self.assertEqual(contents.count("Fd Magnitude:"), nsubjacs)
-        self.assertEqual(contents.count("Absolute Error (Jrev - Jfd), step="), 0)
-        self.assertEqual(contents.count("Absolute Error (Jrev - Jfd)"), nsubjacs)
-        self.assertEqual(contents.count("Relative Error (Jrev - Jfd) / J"), nsubjacs)
+        self.assertEqual(contents.count("fd value:"), nsubjacs)
+        self.assertEqual(contents.count("Max Tolerance Violation (Jrev - Jfd) - (atol + rtol * Jfd), step="), 0)
+        self.assertEqual(contents.count("Max Tolerance Violation (Jrev - Jfd) - (atol + rtol * Jfd)"), nsubjacs)
         self.assertEqual(contents.count("Raw FD Derivative (Jfd), step="), 0)
         self.assertEqual(contents.count("Raw FD Derivative (Jfd)"), nsubjacs)
 
@@ -2088,7 +2045,7 @@ class TestCheckTotalsMultipleSteps(unittest.TestCase):
                 nsubjacs = 18
                 self.assertEqual(contents.count("step"), 0)
                 # check number of rows/cols
-                self.assertEqual(contents.count("+-------------------------------+------------------+-------------+-------------+-------------+-------------+--------------------+"), nsubjacs + 1)
+                self.assertEqual(contents.count("+-----------------------------+----------------+---------------+---------------+------------------------+-------------------+"), nsubjacs + 1)
 
     def test_single_cs_step_compact(self):
         for mode in ('fwd', 'rev'):
@@ -2102,7 +2059,7 @@ class TestCheckTotalsMultipleSteps(unittest.TestCase):
                 nsubjacs = 18
                 self.assertEqual(contents.count("step"), 0)
                 # check number of rows/cols
-                self.assertEqual(contents.count("+-------------------------------+------------------+-------------+-------------+-------------+-------------+------------+"), nsubjacs + 1)
+                self.assertEqual(contents.count("+-----------------------------+----------------+---------------+---------------+------------------------+------------+"), nsubjacs + 1)
 
     def test_multi_fd_steps_fwd(self):
         p = om.Problem(model=CircleOpt(), driver=om.ScipyOptimizeDriver(optimizer='SLSQP', disp=False))
@@ -2110,12 +2067,11 @@ class TestCheckTotalsMultipleSteps(unittest.TestCase):
         p.run_model()
         stream = StringIO()
         p.check_totals(step=[1e-6, 1e-7], out_stream=stream)
-        contents = stream.getvalue()
+        contents = strip_formatting(stream.getvalue())
         nsubjacs = 18
         self.assertEqual(contents.count("Full Model:"), nsubjacs)
-        self.assertEqual(contents.count("Fd Magnitude:"), nsubjacs * 2)
-        self.assertEqual(contents.count("Absolute Error (Jfor - Jfd), step="), nsubjacs * 2)
-        self.assertEqual(contents.count("Relative Error (Jfor - Jfd) / Jf"), nsubjacs * 2)
+        self.assertEqual(contents.count("fd value:"), nsubjacs * 2)
+        self.assertEqual(contents.count("Max Tolerance Violation (Jfwd - Jfd) - (atol + rtol * Jfd), step="), nsubjacs * 2)
         self.assertEqual(contents.count("Raw FD Derivative (Jfd), step="), nsubjacs * 2)
 
     def test_multi_fd_steps_fwd_directional(self):
@@ -2124,11 +2080,10 @@ class TestCheckTotalsMultipleSteps(unittest.TestCase):
         p.run_model()
         stream = StringIO()
         p.check_totals(step=[1e-6, 1e-7], directional=True, out_stream=stream)
-        contents = stream.getvalue()
+        contents = strip_formatting(stream.getvalue())
         self.assertEqual(contents.count("Full Model:"), 3)
-        self.assertEqual(contents.count("Fd Magnitude:"), 6)
-        self.assertEqual(contents.count("Absolute Error (Jfor - Jfd), step="), 6)
-        self.assertEqual(contents.count("Relative Error (Jfor - Jfd) / Jf"), 6)
+        self.assertEqual(contents.count("fd value:"), 6)
+        self.assertEqual(contents.count("Max Tolerance Violation ([fwd, fd] Dot Product Test), step="), 6)
         self.assertEqual(contents.count("Directional FD Derivative (Jfd), step="), 6)
 
     def test_multi_fd_steps_rev(self):
@@ -2137,12 +2092,11 @@ class TestCheckTotalsMultipleSteps(unittest.TestCase):
         p.run_model()
         stream = StringIO()
         p.check_totals(step=[1e-6, 1e-7], out_stream=stream)
-        contents = stream.getvalue()
+        contents = strip_formatting(stream.getvalue())
         nsubjacs = 18
         self.assertEqual(contents.count("Full Model:"), nsubjacs)
-        self.assertEqual(contents.count("Fd Magnitude:"), nsubjacs * 2)
-        self.assertEqual(contents.count("Absolute Error (Jrev - Jfd), step="), nsubjacs * 2)
-        self.assertEqual(contents.count("Relative Error (Jrev - Jfd) / J"), nsubjacs * 2)
+        self.assertEqual(contents.count("fd value:"), nsubjacs * 2)
+        self.assertEqual(contents.count("Max Tolerance Violation (Jrev - Jfd) - (atol + rtol * Jfd), step="), nsubjacs * 2)
         self.assertEqual(contents.count("Raw FD Derivative (Jfd), step="), nsubjacs * 2)
 
     def test_multi_fd_steps_rev_directional(self):
@@ -2151,11 +2105,10 @@ class TestCheckTotalsMultipleSteps(unittest.TestCase):
         p.run_model()
         stream = StringIO()
         p.check_totals(step=[1e-6, 1e-7], directional=True, out_stream=stream)
-        contents = stream.getvalue()
+        contents = strip_formatting(stream.getvalue())
         self.assertEqual(contents.count("Full Model:"), 6)
-        self.assertEqual(contents.count("Fd Magnitude:"), 12)
-        self.assertEqual(contents.count("Absolute Error ([rev, fd] Dot Product Test), step="), 12)
-        self.assertEqual(contents.count("Relative Error ([rev, fd] Dot Product Test) / Jfd, step="), 12)
+        self.assertEqual(contents.count("fd value:"), 12)
+        self.assertEqual(contents.count("Max Tolerance Violation ([rev, fd] Dot Product Test), step="), 12)
         self.assertEqual(contents.count("Directional FD Derivative (Jfd) Dot Product, step="), 12)
 
     def test_multi_fd_steps_compact(self):
@@ -2165,12 +2118,12 @@ class TestCheckTotalsMultipleSteps(unittest.TestCase):
                 p.setup(mode=mode)
                 p.run_model()
                 stream = StringIO()
-                p.check_totals(step=[1e-6, 1e-7], compact_print=True, out_stream=stream)
-                contents = stream.getvalue()
+                p.check_totals(step=[1e-6, 1e-7], compact_print=True, out_stream=stream, abs_err_tol=2e-6, rel_err_tol=3e-6)
+                contents = strip_formatting(stream.getvalue())
                 nsubjacs = 18
                 self.assertEqual(contents.count("step"), 1)
                 # check number of rows/cols
-                self.assertEqual(contents.count("+-------------------------------+------------------+-------------+-------------+-------------+-------------+-------------+--------------------+"), (nsubjacs*2) + 1)
+                self.assertEqual(contents.count("+-----------------------------+----------------+---------------+---------------+---------------+------------------------+------------+"), (nsubjacs*2) + 1)
 
     def test_multi_cs_steps_compact(self):
         for mode in ('fwd', 'rev'):
@@ -2184,7 +2137,7 @@ class TestCheckTotalsMultipleSteps(unittest.TestCase):
                 nsubjacs = 18
                 self.assertEqual(contents.count("step"), 1)
                 # check number of rows/cols
-                self.assertEqual(contents.count("+-------------------------------+------------------+-------------+-------------+-------------+-------------+-------------+------------+"), (nsubjacs*2) + 1)
+                self.assertEqual(contents.count("+-----------------------------+----------------+---------------+---------------+---------------+------------------------+------------+"), (nsubjacs*2)+1)
 
     def test_multi_fd_steps_compact_directional(self):
         expected_divs = {
