@@ -8,7 +8,7 @@ import openmdao.api as om
 from openmdao.test_suite.components.distributed_components import DistribComp, Summer
 from openmdao.utils.mpi import MPI, multi_proc_exception_check
 from openmdao.utils.array_utils import evenly_distrib_idxs, take_nth
-from openmdao.utils.assert_utils import assert_near_equal, assert_warning
+from openmdao.utils.assert_utils import assert_near_equal, assert_warning, assert_check_partials
 from openmdao.utils.om_warnings import DistributedComponentWarning
 from openmdao.utils.variable_table import NA
 
@@ -17,13 +17,6 @@ try:
 except ImportError:
     PETScVector = None
 
-if MPI:
-    rank = MPI.COMM_WORLD.rank
-    commsize = MPI.COMM_WORLD.size
-else:
-    rank = 0
-    commsize = 1
-
 
 class InOutArrayComp(om.ExplicitComponent):
 
@@ -39,11 +32,14 @@ class InOutArrayComp(om.ExplicitComponent):
 
         self.add_input('invec', np.ones(arr_size, float))
         self.add_output('outvec', np.ones(arr_size, float))
-
+        self.declare_partials('outvec', 'invec', rows=np.arange(arr_size), cols=np.arange(arr_size))
     def compute(self, inputs, outputs):
         time.sleep(self.options['delay'])
         outputs['outvec'] = inputs['invec'] * 2.
 
+    def compute_partials(self, inputs, partials):
+        partials['outvec', 'invec'] = 2.0 * np.ones((self.options['arr_size'],))
+
 
 class DistribCompSimple(om.ExplicitComponent):
     """Uses 2 procs but takes full input vars"""
@@ -59,10 +55,10 @@ class DistribCompSimple(om.ExplicitComponent):
         self.add_output('outvec', np.ones(arr_size, float), distributed=True)
 
     def compute(self, inputs, outputs):
-        if MPI and self.comm != MPI.COMM_NULL:
-            if rank == 0:
+        if MPI and self.comm.size > 1:
+            if self.comm.rank == 0:
                 outvec = inputs['invec'] * 0.25
-            elif rank == 1:
+            else:
                 outvec = inputs['invec'] * 0.5
 
             # now combine vecs from different processes
@@ -129,9 +125,6 @@ class DistribOverlappingInputComp(om.ExplicitComponent):
         """ component declares the local sizes and sets initial values
         for all distributed inputs and outputs"""
 
-        comm = self.comm
-        rank = comm.rank
-
         arr_size = self.options['arr_size']
         local_size = self.options['local_size']
 
@@ -282,7 +275,7 @@ class NOMPITests(unittest.TestCase):
 
         p = om.Problem()
         top = p.model
-        C1 = top.add_subsystem("C1", InOutArrayComp(arr_size=size))
+        top.add_subsystem("C1", InOutArrayComp(arr_size=size))
         C2 = top.add_subsystem("C2", DistribInputComp(arr_size=size))
         top.connect('C1.outvec', 'C2.invec')
 
@@ -341,15 +334,15 @@ class DistributedIO(unittest.TestCase):
     N_PROCS = 2
 
     def test_driver_metadata(self):
-        self.comm = MPI.COMM_WORLD
-
         p = om.Problem()
+        comm = p.comm
+
         d_ivc = p.model.add_subsystem('distrib_ivc',
                                     om.IndepVarComp(distributed=True),
                                     promotes=['*'])
 
         # Sending different values to different ranks
-        if self.comm.rank == 0:
+        if comm.rank == 0:
             ndvs = 3
         else:
             ndvs = 2
@@ -429,7 +422,7 @@ class MPITests(unittest.TestCase):
 
         p = om.Problem()
         top = p.model
-        C1 = top.add_subsystem("C1", InOutArrayComp(arr_size=size))
+        top.add_subsystem("C1", InOutArrayComp(arr_size=size))
         C2 = top.add_subsystem("C2", DistribCompSimple(arr_size=size))
         top.connect('C1.outvec', 'C2.invec')
 
@@ -450,8 +443,8 @@ class MPITests(unittest.TestCase):
 
         p = om.Problem()
         top = p.model
-        C1 = top.add_subsystem("C1", InOutArrayComp(arr_size=size))
-        C2 = top.add_subsystem("C2", DistribCompWithDerivs(arr_size=size))
+        top.add_subsystem("C1", InOutArrayComp(arr_size=size))
+        top.add_subsystem("C2", DistribCompWithDerivs(arr_size=size))
         top.connect('C1.outvec', 'C2.invec')
 
         p.setup()
@@ -463,9 +456,7 @@ class MPITests(unittest.TestCase):
         p.run_model()
 
         # this used to fail (bug #1279)
-        cpd = p.check_partials(out_stream=None)
-        for (of, wrt) in cpd['C2']:
-            np.testing.assert_allclose(cpd['C2'][of, wrt]['rel error'][0], 0.0, atol=1e-9)
+        assert_check_partials(p.check_partials(out_stream=None))
 
     def test_list_inputs_outputs(self):
         size = 11
@@ -474,8 +465,8 @@ class MPITests(unittest.TestCase):
 
         class Model(om.Group):
             def setup(self):
-                C1 = self.add_subsystem("C1", InOutArrayComp(arr_size=size))
-                C2 = self.add_subsystem("C2", DistribCompSimple(arr_size=size))
+                self.add_subsystem("C1", InOutArrayComp(arr_size=size))
+                self.add_subsystem("C2", DistribCompSimple(arr_size=size))
                 self.connect('C1.outvec', 'C2.invec')
 
             def configure(self):
@@ -598,6 +589,7 @@ class MPITests(unittest.TestCase):
 
         p = om.Problem(Model())
         p.setup()
+        p.final_setup()
 
         # verify list_inputs/list_outputs work before final_setup for distributed comp on rank 0 only
         inputs = p.model.C2.list_inputs(shape=True, global_shape=True, val=True, out_stream=None)
@@ -609,8 +601,6 @@ class MPITests(unittest.TestCase):
         outputs = p.model.C2.list_outputs(shape=True, global_shape=True, val=True, all_procs=True, out_stream=None)
         verify(inputs, outputs, pathnames=False, comm=p.comm, final=True)
 
-        p.final_setup()
-
         p['C1.invec'] = np.ones(size, float) * 5.0
 
         # verify list_inputs/list_outputs work before run for distributed comp on rank 0 only
@@ -640,7 +630,7 @@ class MPITests(unittest.TestCase):
 
         p = om.Problem()
         top = p.model
-        C1 = top.add_subsystem("C1", InOutArrayComp(arr_size=size))
+        top.add_subsystem("C1", InOutArrayComp(arr_size=size))
         C2 = top.add_subsystem("C2", DistribInputComp(arr_size=size))
         top.connect('C1.outvec', 'C2.invec')
 
@@ -661,10 +651,10 @@ class MPITests(unittest.TestCase):
         p = om.Problem()
 
         top = p.model
-        C1 = top.add_subsystem("C1", InOutArrayComp(arr_size=size))
+        top.add_subsystem("C1", InOutArrayComp(arr_size=size))
         C2 = top.add_subsystem("C2", DistribInputComp(arr_size=size))
-        C3 = top.add_subsystem("C3", om.ExecComp("y=x", x=np.zeros(size*commsize),
-                                                 y=np.zeros(size*commsize)))
+        top.add_subsystem("C3", om.ExecComp("y=x", x=np.zeros(size*p.comm.size),
+                                                 y=np.zeros(size*p.comm.size)))
 
         comm = p.comm
         rank = comm.rank
@@ -692,8 +682,8 @@ class MPITests(unittest.TestCase):
 
         p = om.Problem()
         top = p.model
-        C1 = top.add_subsystem("C1", InOutArrayComp(arr_size=size))
-        C2 = top.add_subsystem("C2", DistribInputDistribOutputComp(arr_size=size))
+        top.add_subsystem("C1", InOutArrayComp(arr_size=size))
+        top.add_subsystem("C2", DistribInputDistribOutputComp(arr_size=size))
         C3 = top.add_subsystem("C3", DistribGatherComp(arr_size=size))
         top.connect('C1.outvec', 'C2.invec')
         top.connect('C2.outvec', 'C3.invec')
@@ -716,7 +706,7 @@ class MPITests(unittest.TestCase):
         top = p.model
         comm = p.comm
 
-        idxs = list(take_nth(rank, comm.size, range(size)))
+        idxs = list(take_nth(p.comm.rank, comm.size, range(size)))
 
         C1 = top.add_subsystem("C1", InOutArrayComp(arr_size=size))
         C2 = top.add_subsystem("C2", DistribNoncontiguousComp(size=len(idxs)))
@@ -746,8 +736,10 @@ class MPITests(unittest.TestCase):
         # entries are distributed to multiple processes
         size = 11
 
+        p = om.Problem()
+
         # need to initialize the input to have the correct local size
-        if rank == 0:
+        if p.comm.rank == 0:
             local_size = 8
             start = 0
             end = 8
@@ -756,9 +748,8 @@ class MPITests(unittest.TestCase):
             start = 4
             end = 11
 
-        p = om.Problem()
         top = p.model
-        C1 = top.add_subsystem("C1", InOutArrayComp(arr_size=size))
+        top.add_subsystem("C1", InOutArrayComp(arr_size=size))
         C2 = top.add_subsystem("C2", DistribOverlappingInputComp(arr_size=size, local_size=local_size))
         top.connect('C1.outvec', 'C2.invec', src_indices=np.arange(start, end, dtype=int))
         p.setup()
@@ -796,8 +787,8 @@ class MPITests(unittest.TestCase):
 
         p = om.Problem()
         top = p.model
-        C1 = top.add_subsystem("C1", InOutArrayComp(arr_size=size))
-        C2 = top.add_subsystem("C2", DistribInputDistribOutputComp(arr_size=size))
+        top.add_subsystem("C1", InOutArrayComp(arr_size=size))
+        top.add_subsystem("C2", DistribInputDistribOutputComp(arr_size=size))
         C3 = top.add_subsystem("C3", NonDistribGatherComp(size=size))
         top.connect('C1.outvec', 'C2.invec')
         top.connect('C2.outvec', 'C3.invec', om.slicer[:])
@@ -817,15 +808,56 @@ class MPITests(unittest.TestCase):
         size = 2
 
         prob = om.Problem()
-        C2 = prob.model.add_subsystem("C", DistribCompSimple(arr_size=size))
+        prob.model.add_subsystem("C", DistribCompSimple(arr_size=size))
 
         with self.assertRaises(RuntimeError) as context:
             prob.setup()
-
-        msg = 'Distributed component input "C.invec" requires an IndepVarComp.'
+            prob.final_setup()
 
         err_msg = str(context.exception).split(':')[-1]
-        self.assertEqual(err_msg, msg)
+        self.assertEqual(err_msg, 'Distributed component input "C.invec" is not connected.')
+
+    def test_auto_ivc_error_promoted(self):
+        size = 2
+
+        prob = om.Problem()
+        prob.model.add_subsystem("C", DistribCompSimple(arr_size=size), promotes=['*'])
+        prob.setup()
+
+        with self.assertRaises(RuntimeError) as context:
+            prob.final_setup()
+
+        err_msg = str(context.exception).split(':')[-1]
+        self.assertEqual(err_msg, 'Distributed component input "C.invec", promoted as "invec", is not connected.')
+
+    def test_bad_distrib_connect(self):
+        class Adder(om.ExplicitComponent):
+            def setup(self):
+                self.add_input('x', shape_by_conn=True, distributed=True)
+                self.add_output('x_sum', shape=1)
+
+            def compute(self, inputs, outputs):
+                outputs['x_sum'] = np.sum(inputs['x'])
+
+        prob = om.Problem(name='bad_distrib_problem')
+        ivc = prob.model.add_subsystem('ivc',om.IndepVarComp())
+        ivc.add_output('x', val = np.ones(10), distributed=True)
+
+        prob.model.add_subsystem('adder', Adder())
+
+        prob.model.connect('ivc.x0','adder.x')
+        prob.setup()
+        try:
+            prob.final_setup()
+        except Exception as err:
+            self.assertTrue(
+                "\nCollected errors for problem 'bad_distrib_problem':"
+                "\n   <model> <class Group>: Attempted to connect from 'ivc.x0' to 'adder.x', but "
+                "'ivc.x0' doesn't exist. Perhaps you meant to connect to one of the following outputs: ['ivc.x']."
+                "\n   <model> <class Group>: Failed to resolve shapes for ['adder.x']. To see the "
+                "dynamic shape dependency graph, do 'openmdao view_dyn_shapes <your_py_file>'." in str(err))
+        else:
+            self.fail("Exception expected.")
 
 
 class NonParallelTests(unittest.TestCase):
@@ -871,8 +903,8 @@ class ProbRemoteTests(unittest.TestCase):
         top.connect('P.invec1', 'par.C1.invec')
         top.connect('P.invec2', 'par.C2.invec')
 
-        C1 = par.add_subsystem("C1", DistribInputDistribOutputComp(arr_size=size))
-        C2 = par.add_subsystem("C2", DistribInputDistribOutputComp(arr_size=size))
+        par.add_subsystem("C1", DistribInputDistribOutputComp(arr_size=size))
+        par.add_subsystem("C2", DistribInputDistribOutputComp(arr_size=size))
 
         p.setup()
 
@@ -962,7 +994,7 @@ class ProbRemoteTests(unittest.TestCase):
         top.connect('P.invec', 'C1.invec')
         top.connect('P.disc_in', 'C1.disc_in')
 
-        C1 = top.add_subsystem("C1", DistribInputDistribOutputDiscreteComp(arr_size=size))
+        top.add_subsystem("C1", DistribInputDistribOutputDiscreteComp(arr_size=size))
         p.setup()
 
         # Conclude setup but don't run model.
