@@ -2,48 +2,27 @@
 import os
 import re
 import sys
-import types
+import textwrap
+from types import TracebackType
 import unittest
 from contextlib import contextmanager
 from fnmatch import fnmatchcase
 from io import StringIO
 from numbers import Integral
-from inspect import currentframe, getouterframes
+from inspect import currentframe, getouterframes, stack, isfunction, ismethod
 
 from collections.abc import Iterable
 
 import numpy as np
 
-from openmdao.core.constants import INF_BOUND
-from openmdao.utils.om_warnings import issue_warning
+from openmdao.core.constants import INF_BOUND, _UNDEFINED
 from openmdao.utils.array_utils import shape_to_len
 
 
 _float_inf = float('inf')
 
 
-def _convert_auto_ivc_to_conn_name(conns_dict, name):
-    """
-    Convert name of auto_ivc val to promoted input name.
-
-    Parameters
-    ----------
-    conns_dict : dict
-        Dictionary of global connections.
-    name : str
-        Name of auto_ivc to be found.
-
-    Returns
-    -------
-    str
-        Promoted input name.
-    """
-    for key, val in conns_dict.items():
-        if val == name:
-            return key
-
-
-def ensure_compatible(name, value, shape=None, indices=None):
+def ensure_compatible(name, value, shape=None, indices=None, default_shape=(1,)):
     """
     Make value compatible with the specified shape or the shape of indices.
 
@@ -57,6 +36,8 @@ def ensure_compatible(name, value, shape=None, indices=None):
         The expected or desired shape of the value.
     indices : Indexer or None
         The indices into a source variable.
+    default_shape : tuple
+        The default shape to use if shape is not provided.
 
     Returns
     -------
@@ -100,12 +81,18 @@ def ensure_compatible(name, value, shape=None, indices=None):
 
     if shape is None:
         # shape is not determined, assume the shape of value was intended
-        value = np.atleast_1d(value)
+        if np.isscalar(value):
+            value = np.full(default_shape, value)
+        else:
+            value = np.asarray(value).reshape(default_shape)
         shape = value.shape
     else:
         # shape is determined, if value is scalar assign it to array of shape
         # otherwise make sure value is an array of the determined shape
-        if np.ndim(value) == 0 or value.shape == (1,):
+        if np.ndim(value) == 0:
+            if shape != ():
+                value = np.full(shape, value)
+        elif value.shape == (1,):
             value = np.full(shape, value)
         else:
             value = np.atleast_1d(value).astype(np.float64)
@@ -116,6 +103,43 @@ def ensure_compatible(name, value, shape=None, indices=None):
     return value, shape
 
 
+def _subjac_meta2value(meta):
+    """
+    Convert subjacobian metadata to value, rows, cols.
+
+    Parameters
+    ----------
+    meta : dict
+        Metadata dict.
+
+    Returns
+    -------
+    ndarray
+        Value of the subjacobian.
+    ndarray or None
+        Row indices of nonzero values in subjacobian.
+    ndarray or None
+        Column indices of nonzero values in subjacobian.
+    """
+    val = meta['val'] if 'val' in meta else None
+    rows = meta['rows'] if 'rows' in meta else None
+    cols = meta['cols'] if 'cols' in meta else None
+
+    if rows is not None:
+        if val is not None and np.isscalar(val):
+            val = np.full(len(rows), val)
+    elif np.isscalar(val):
+        shape = meta['shape'] if 'shape' in meta else None
+        if shape is not None:
+            val = np.full(shape, val)
+        else:
+            val = np.atleast_2d(val)
+    elif val is not None:
+        val = np.atleast_2d(val)
+
+    return val, rows, cols
+
+
 def determine_adder_scaler(ref0, ref, adder, scaler):
     r"""
     Determine proper values of adder and scaler based on user arguments.
@@ -340,9 +364,9 @@ def all_ancestors(pathname, delim='.'):
     ------
     str
     """
-    parts = pathname.split(delim)
-    for i in range(len(parts), 0, -1):
-        yield delim.join(parts[:i])
+    while pathname:
+        yield pathname
+        pathname, _, _ = pathname.rpartition(delim)
 
 
 def find_matches(pattern, var_list):
@@ -363,11 +387,46 @@ def find_matches(pattern, var_list):
     """
     if pattern == '*':
         return var_list
-    elif pattern in var_list:
-        return [pattern]
     return [name for name in var_list if fnmatchcase(name, pattern)]
 
 
+def pattern_filter(patterns, var_iter, name_index=None):
+    """
+    Yield variable names that match a given pattern.
+
+    Parameters
+    ----------
+    patterns : iter of str
+        Glob patterns or variable names.
+    var_iter : iter of str or iter of tuple/list
+        Iterator of variable names (or tuples containing variable names) to search for patterns.
+    name_index : int or None
+        If not None, the var_iter is assumed to yield tuples, and the
+        name_index is the index of the variable name in the tuple.
+
+    Yields
+    ------
+    str
+        Variable name or corresponding tuple where the name matches a pattern.
+    """
+    if '*' in patterns:
+        yield from var_iter
+    else:
+        if name_index is None:
+            for vname in var_iter:
+                for pattern in patterns:
+                    if fnmatchcase(vname, pattern):
+                        yield vname
+                        break
+        else:
+            for tup in var_iter:
+                vname = tup[name_index]
+                for pattern in patterns:
+                    if fnmatchcase(vname, pattern):
+                        yield tup
+                        break
+
+
 def _find_dict_meta(dct, key):
     """
     Return True if the given key is found in any metadata values in the given dict.
@@ -415,11 +474,57 @@ def pad_name(name, width=10, quotes=False):
         return f"{name}"
 
 
-def add_border(msg, borderstr='=', vpad=0):
+def get_max_widths(rows):
     """
-    Add border lines before and after a message.
+    Determine the maximum width of each column.
 
-    The message is assumed not to span multiple lines.
+    Parameters
+    ----------
+    rows : list of list of str
+        List of rows, where each row is a list of strings.
+
+    Returns
+    -------
+    list of int
+        List of maximum widths for each column.
+    """
+    if not rows:
+        return []
+
+    for irow, row in enumerate(rows):
+        if irow == 0:
+            widths = [len(val) for val in row]
+        else:
+            for i, val in enumerate(row):
+                widths[i] = max(widths[i], len(val))
+    return widths
+
+
+def strs2row_iter(strs, colwidths, delim=' '):
+    """
+    Yield rows of strings formatted into columns.
+
+    Parameters
+    ----------
+    strs : list of str
+        List of strings to be formatted into columns.
+    colwidths : list of int
+        List of column widths.
+    delim : str
+        Delimiter to use between columns.
+
+    Yields
+    ------
+    str
+        Formatted row of strings.
+    """
+    for row in strs:
+        yield delim.join(f"{val:<{width}}" for val, width in zip(row, colwidths))
+
+
+def add_border(msg, borderstr='=', vpad=0, above=True, below=True):
+    """
+    Add border lines before and/or after a message.
 
     Parameters
     ----------
@@ -428,18 +533,26 @@ def add_border(msg, borderstr='=', vpad=0):
     borderstr : str
         The repeating string to be used in the border.
     vpad : int
-        The number of blank lines between the border and the message (before and after).
+        The number of blank lines between the border(s) and the message.
+    above : bool
+        If True, add a border above the message.
+    below : bool
+        If True, add a border below the message.
 
     Returns
     -------
     str
-        A string containing the original message enclosed in a border.
+        A string containing the original message and border(s) before and/or after.
     """
-    border = len(msg) * borderstr
+    width = max(len(line) for line in msg.split('\n'))
+    border = width * borderstr
     # handle borderstr of more than 1 char
-    border = border[:len(msg)]
-    padding = '\n' * (vpad + 1)
-    return f"{border}{padding}{msg}{padding}{border}"
+    border = border[:width]
+    uborder = border if above else ''
+    lborder = border if below else ''
+    upadding = '\n' * (vpad + 1) if uborder else '\n' * vpad
+    lpadding = '\n' * vpad if vpad else '\n'
+    return f"{uborder}{upadding}{msg}{lpadding}{lborder}"
 
 
 def run_model(prob, ignore_exception=False):
@@ -494,7 +607,7 @@ def run_driver(prob):
 
     sys.stdout = strout
     try:
-        failed = prob.run_driver()
+        failed = not prob.run_driver().success
     finally:
         sys.stdout = stdout
 
@@ -546,6 +659,48 @@ def printoptions(*args, **kwds):
         np.set_printoptions(**opts)
 
 
+@contextmanager
+def indent_context(stream, indent='   '):
+    """
+    Context manager for indenting all std output.
+
+    Parameters
+    ----------
+    stream : stream
+        The stream to write indented output to.
+    indent : str
+        The string to use for indentation.
+
+    Yields
+    ------
+    str
+        The current stdout.
+    """
+    save_stdout = sys.stdout
+    save_stderr = sys.stderr
+
+    # buffer all of stdout so we can indent it all
+    sys.stdout = StringIO()
+    sys.stderr = sys.stdout
+
+    try:
+        yield sys.stdout
+    except Exception:
+        # not sure what happened, so just print the whole thing without indentation
+        print(sys.stdout.getvalue(), file=save_stdout)
+        errs = sys.stderr.getvalue()
+        if errs:
+            print(errs, file=save_stderr)
+        raise
+    else:
+        # nothing went wrong so do indentation
+        if stream is not None:
+            stream.write(textwrap.indent(sys.stdout.getvalue(), indent))
+    finally:
+        sys.stderr = save_stderr
+        sys.stdout = save_stdout
+
+
 def _nothing():
     yield None
 
@@ -596,27 +751,6 @@ def remove_whitespace(s, right=False, left=False):
         return re.sub(r"^\s+", "", s, flags=re.UNICODE)
 
 
-_badtab = r'`~@#$%^&*()[]{}-+=|\/?<>,.:;'
-_transtab = str.maketrans(_badtab, '_' * len(_badtab))
-
-
-def str2valid_python_name(s):
-    """
-    Translate a given string into a valid python variable name.
-
-    Parameters
-    ----------
-    s : str
-        The string to be translated.
-
-    Returns
-    -------
-    str
-        The valid python name string.
-    """
-    return s.translate(_transtab)
-
-
 _container_classes = (list, tuple, set)
 
 
@@ -639,9 +773,7 @@ def make_serializable(o):
     if isinstance(o, _container_classes):
         return [make_serializable(item) for item in o]
     elif isinstance(o, dict):
-        s_key = [make_serializable_key(item) for item in o.keys()]
-        s_val = [make_serializable(item) for item in o.values()]
-        return dict(zip(s_key, s_val))
+        return {make_serializable_key(k): make_serializable(v) for k, v in o.items()}
     elif isinstance(o, np.ndarray):
         return o.tolist()
     elif isinstance(o, np.number):
@@ -707,9 +839,7 @@ def default_noraise(o):
     if isinstance(o, _container_classes):
         return [default_noraise(item) for item in o]
     elif isinstance(o, dict):
-        s_key = [make_serializable_key(item) for item in o.keys()]
-        s_val = [default_noraise(item) for item in o.values()]
-        return dict(zip(s_key, s_val))
+        return {make_serializable_key(k): default_noraise(v) for k, v in o.items()}
     elif isinstance(o, np.ndarray):
         return o.tolist()
     elif isinstance(o, np.number):
@@ -802,30 +932,6 @@ def match_includes_excludes(name, includes=None, excludes=None):
     return False
 
 
-def filtered_name_iter(name_iter, includes=None, excludes=None):
-    """
-    Yield names that pass through the includes and excludes filters.
-
-    Parameters
-    ----------
-    name_iter : iter of str
-        Iterator over names to be checked for match.
-    includes : iter of str or None
-        Glob patterns for name to include in the filtering.  None, the default, means
-        include all.
-    excludes : iter of str or None
-        Glob patterns for name to exclude in the filtering.
-
-    Yields
-    ------
-    str
-        Each name that passes through the filters.
-    """
-    for name in name_iter:
-        if match_includes_excludes(name, includes, excludes):
-            yield name
-
-
 def meta2src_iter(meta_iter):
     """
     Yield the source name for each metadata dict in the given iterator.
@@ -1012,14 +1118,6 @@ def _src_name_iter(proms):
         yield meta['source']
 
 
-def _src_or_alias_name(meta):
-    if 'alias' in meta:
-        alias = meta['alias']
-        if alias:
-            return alias
-    return meta['source']
-
-
 def _src_or_alias_item_iter(proms):
     """
     Yield items from proms dict with promoted input names converted to source or alias names.
@@ -1091,6 +1189,29 @@ def convert_src_inds(parent_src_inds, parent_src_shape, my_src_inds, my_src_shap
         return parent_src_inds.shaped_array(flat=False).reshape(my_src_shape)[my_src_inds()]
 
 
+def is_undefined(obj):
+    """
+    Return True if the object is _UNDEFINED.
+
+    This function should be used instead of `{obj} is _UNDEFINED`, which
+    is not reliable across processes. The use of `{obj} == _UNDEFINED` will
+    fail if `obj` is an array.
+
+    Parameters
+    ----------
+    obj : any
+        Any python object.
+
+    Returns
+    -------
+    bool
+        True if the obj is not an array, and obj == _UNDEFINED.
+    """
+    if isinstance(obj, Iterable):
+        return False
+    return obj == _UNDEFINED
+
+
 def shape2tuple(shape):
     """
     Return shape as a tuple.
@@ -1142,16 +1263,16 @@ def get_connection_owner(system, tgt):
 
     model = system._problem_meta['model_ref']()
     src = model._conn_global_abs_in2out[tgt]
-    abs2prom = model._var_allprocs_abs2prom
+    resolver = model._resolver
 
-    if src in abs2prom['output'] and tgt in abs2prom['input'][tgt]:
-        if abs2prom['input'][tgt] != abs2prom['output'][src]:
+    if resolver.is_abs(src, 'output') and resolver.is_abs(tgt, 'input'):
+        if resolver.abs2prom(tgt, 'input') != resolver.abs2prom(src, 'output'):
             # connection is explicit
             for g in model.system_iter(include_self=True, recurse=True, typ=Group):
                 if g._manual_connections:
-                    tprom = g._var_allprocs_abs2prom['input'][tgt]
+                    tprom = g._resolver.abs2prom(tgt, 'input')
                     if tprom in g._manual_connections:
-                        return g, g._var_allprocs_abs2prom['output'][src], tprom
+                        return g, g._resolver.abs2prom(src, 'output'), tprom
 
     return system, src, tgt
 
@@ -1169,7 +1290,7 @@ def wing_dbg():
         new = sys.path[:] + [os.environ['WINGHOME']]
         sys.path = new
         try:
-            import wingdbstub
+            import wingdbstub  # noqa: F401
         finally:
             sys.path = save
 
@@ -1221,12 +1342,12 @@ class LocalRangeIterable(object):
         all_abs2meta = system._var_allprocs_abs2meta['output']
         if vname in all_abs2meta:
             sizes = system._var_sizes['output']
-            slices = system._outputs.get_slice_dict()
+            vec = system._outputs
             abs2meta = system._var_abs2meta['output']
         else:
             all_abs2meta = system._var_allprocs_abs2meta['input']
             sizes = system._var_sizes['input']
-            slices = system._inputs.get_slice_dict()
+            vec = system._inputs
             abs2meta = system._var_abs2meta['input']
 
         if all_abs2meta[vname]['distributed']:
@@ -1243,13 +1364,14 @@ class LocalRangeIterable(object):
             self._var_size = all_abs2meta[vname]['global_size']
         else:
             self._iter = self._serial_iter
+            start, stop = vec.get_range(vname)
             if use_vec_offset:
-                self._inds = range(slices[vname].start, slices[vname].stop)
+                self._inds = range(start, stop)
             else:
-                self._inds = range(slices[vname].stop - slices[vname].start)
+                self._inds = range(stop - start)
             self._var_size = all_abs2meta[vname]['global_size']
 
-    def __str__(self):
+    def __repr__(self):
         """
         Return a string representation of the iterator.
 
@@ -1329,7 +1451,7 @@ def make_traceback():
         The newly constructed traceback.
     """
     finfo = getouterframes(currentframe())[2]
-    return types.TracebackType(None, finfo.frame, finfo.frame.f_lasti, finfo.frame.f_lineno)
+    return TracebackType(None, finfo.frame, finfo.frame.f_lasti, finfo.frame.f_lineno)
 
 
 if env_truthy('OM_DBG'):
@@ -1427,3 +1549,340 @@ def get_rev_conns(conns):
         else:
             rev[src] = [tgt]
     return rev
+
+
+def vprint(it, end='\n', getter=None, file=None):
+    """
+    Iterate over the given iterator and print each item separated by end.
+
+    Parameters
+    ----------
+    it : iter
+        Iterator to be printed.
+    end : str
+        String written after each item.
+    getter : function or None
+        If not None, only print the part of each item returned by getter(item).
+    file : file-like or None
+        File to write to.  If None, use sys.stdout.
+    """
+    if file is None:
+        file = sys.stdout
+
+    for val in it:
+        if getter is not None:
+            val = getter(val)
+        print(val, end=end, file=file)
+
+
+def _default_predicate(name, obj):
+    """
+    Determine if a given method should be traced.
+
+    Parameters
+    ----------
+    name : str
+        Name of the method.
+    obj : object
+        The object being checked.
+
+    Returns
+    -------
+    bool
+        True if the method should be traced.
+    """
+    for n in ['solve', 'apply', 'compute', 'setup', 'coloring', 'linearize', 'get_outputs_dir',
+              'approx', 'static', 'get_vars', 'abs_get']:
+        if n in name:
+            return True
+    return False
+
+
+_trace_predicate = _default_predicate
+
+
+def set_trace_predicate(funct):
+    """
+    Set the function that determines which methods will be traced.
+
+    Parameters
+    ----------
+    funct : function
+        Set of functions to be traced.
+    """
+    global _trace_predicate
+    _trace_predicate = funct
+
+
+def _decorate_functs(attrs, predicate, decorator):
+    """
+    Decorate the functions in the given attribute dict.
+
+    Parameters
+    ----------
+    cname : str
+        The name of the class containing the functions.
+    attrs : dict
+        The attribute dict containing the functions to be decorated.
+    predicate : function
+        Function returning True if the function should be decorated.
+    decorator : function
+        The decorator function.
+    """
+    for name, obj in attrs.items():
+        if (isfunction(obj) or ismethod(obj)) and predicate(name, obj):
+            attrs[name] = decorator(obj)
+
+
+SystemMetaclass = type
+ProblemMetaclass = type
+SolverMetaclass = type
+DriverMetaclass = type
+DebugMeta = type
+
+
+def om_dump(*args, **kwargs):
+    r"""
+    Do nothing.
+
+    Parameters
+    ----------
+    *args : list
+        Positional args.
+    **kwargs : dict
+        Named args.
+    """
+    pass
+
+
+def dbg(funct):
+    """
+    Do nothing.
+
+    Parameters
+    ----------
+    funct : function
+        The function being decorated.
+
+    Returns
+    -------
+    function
+        The function.
+    """
+    return funct
+
+
+def _wrap_comm(comm, scope=None):
+    return comm
+
+
+def _unwrap_comm(comm):
+    return comm
+
+
+_om_dump = env_truthy('OPENMDAO_DUMP')
+# OPENMDAO_DUMP can have values like 'stdout', 'stderr', 'rank', 'pid', 'rank,pid', 'pid,rank'
+# 'rank' means to include the rank in the dump file name, e.g., om_dump_0.out
+# 'pid' means to include the pid in the dump file name, e.g., om_dump_12345.out
+# if rank and pid are both included, the file name will be, e.g., om_dump_0_12345.out
+# 'stdout' means to dump to stdout (so rank and pid are ignored)
+# 'stderr' means to dump to stderr (so rank and pid are ignored)
+# 'trace' means to print function entry and exit
+# 'args' means to print function entry and exit with args and kwargs if trace is also included
+# if OPENMDAO_DUMP is just a plain truthy value, like '1', then we dump to a file
+# named om_dump.out.
+
+if _om_dump:
+    parts = [s.strip() for s in os.environ['OPENMDAO_DUMP'].split(',')]
+    trace = 'trace' in parts
+
+    if 'stdout' in parts:
+        _dump_stream = sys.stdout
+    elif 'stderr' in parts:
+        _dump_stream = sys.stderr
+    else:
+        rankstr = pidstr = ''
+        if 'rank' in parts:
+            from openmdao.utils.mpi import MPI
+            rankstr = f"_{MPI.COMM_WORLD.rank if MPI else 0}"
+
+        if 'pid' in parts:
+            pidstr = f"_{os.getpid()}"
+
+        _dump_stream = open(f'om_dump{rankstr}{pidstr}.out', 'w')
+
+    _show_args = 'args' in parts
+
+    def om_dump(*args, **kwargs):
+        """
+        Dump to a stream if OPENMDAO_DUMP is truthy in the environment.
+
+        Depending on the value of OPENMDAO_DUMP, output will go to file(s), stdout, or stderr.
+
+        Parameters
+        ----------
+        args : list
+            Positional args.
+        kwargs : dict
+            Named args.
+        """
+        kwargs['file'] = _dump_stream
+        kwargs['flush'] = True
+        print(*args, **kwargs)
+
+    def dbg(cname):
+        """
+        Decorate function to print function entry and exit.
+
+        Parameters
+        ----------
+        cname : str
+            The name of the class containing the function.
+
+        Returns
+        -------
+        function
+            The decorated function.
+        """
+        def _dbg(funct):
+            def wrapper(*args, **kwargs):
+                try:
+                    path = args[0].pathname + '.'
+                except Exception:
+                    path = ''
+                indent = call_depth2indent()
+                if _show_args:
+                    argstr = f"(args={args}, kwargs={kwargs})"
+                else:
+                    argstr = ''
+                om_dump(f"{indent}--> {cname}:{path}{funct.__name__}{argstr}")
+                ret = funct(*args, **kwargs)
+                om_dump(f"{indent}<-- {cname}:{path}{funct.__name__}")
+                return ret
+
+            return wrapper
+        return _dbg
+
+    if trace:
+        class DebugMeta(type):
+            """
+            A metaclass to add trace output to some methods of the class.
+
+            Parameters
+            ----------
+            name : str
+                The name of the class.
+            bases : tuple
+                The base classes of the class.
+            attrs : dict
+                The attributes of the class.
+
+            Returns
+            -------
+            class
+                The class with the metaclass applied.
+            """
+
+            def __new__(metaclass, name, bases, attrs):
+                """
+                Add trace output to some methods of the class.
+
+                Parameters
+                ----------
+                name : str
+                    The name of the class.
+                bases : tuple
+                    The base classes of the class.
+                attrs : dict
+                    The attributes of the class.
+
+                Returns
+                -------
+                class
+                    The class with trace output added to some methods
+                """
+                _decorate_functs(attrs, _trace_predicate, dbg(name))
+                return super().__new__(metaclass, name, bases, attrs)
+
+        SystemMetaclass = DebugMeta
+        ProblemMetaclass = DebugMeta
+        SolverMetaclass = DebugMeta
+        DriverMetaclass = DebugMeta
+
+        def _comm_debug_decorator(fn, scope):  # pragma no cover
+            def _wrap(*args, **kwargs):
+                sc = '' if scope is None else f"{scope}."
+                indent = call_depth2indent()
+                if _show_args:
+                    argstr = f"(args={args}, kwargs={kwargs})"
+                else:
+                    argstr = ''
+                om_dump(f"{indent}--> {sc}{fn.__name__}{argstr}")
+                ret = fn(*args, **kwargs)
+                om_dump(f"{indent}<-- {sc}{fn.__name__}")
+                return ret
+            return _wrap
+
+        class _DebugComm(object):  # pragma no cover
+            """
+            Debugging wrapper for an MPI communicator.
+            """
+
+            def __init__(self, comm, scope):
+                if isinstance(comm, _DebugComm):
+                    self.__dict__['_comm'] = comm._comm
+                else:
+                    self.__dict__['_comm'] = comm
+                self.__dict__['_scope'] = scope
+                for name in ['bcast', 'Bcast', 'gather', 'Gather', 'scatter', 'Scatter',
+                             'allgather', 'Allgather', 'Allgatherv', 'allreduce', 'Allreduce',
+                             'send', 'Send', 'recv', 'Recv', 'sendrecv', 'Sendrecv']:
+                    self.__dict__[name] = _comm_debug_decorator(getattr(self._comm, name), scope)
+
+            def __getattr__(self, name):
+                return getattr(self._comm, name)
+
+            def __setattr__(self, name, val):
+                setattr(self._comm, name, val)
+
+        def _wrap_comm(comm, scope=None):  # pragma no cover
+            return _DebugComm(comm, scope)
+
+        def _unwrap_comm(comm):  # pragma no cover
+            if isinstance(comm, _DebugComm):
+                return comm._comm
+            return comm
+
+
+def call_depth2indent(tabsize=2, offset=-1):
+    """
+    Return a string of spaces corresponding to the current call depth.
+
+    Parameters
+    ----------
+    tabsize : int
+        Number of spaces per tab.
+    offset : int
+        Offset to add to the call depth.
+
+    Returns
+    -------
+    str
+        A string of spaces.
+    """
+    return ' ' * ((len(stack()) + offset) * tabsize)
+
+
+def print_with_line_numbers(text, **kwargs):
+    """
+    Print a string with each line preceded by its line number.
+
+    Parameters
+    ----------
+    text : str
+        The text to print with line numbers.
+    **kwargs : dict
+        Keyword arguments to pass to print.
+    """
+    for i, line in enumerate(text.splitlines(), 1):
+        print(f"{i:5d} | {line}", **kwargs)
